/******************************************************************************/
/* AEL Sarl : Web hosting, datacenter, gestion et comptabilité déportées      */
/* Copyright (C) 2008 Affaires En Ligne S.A.R.L.                              */
/* Contact : contact@affaires-en-ligne.fr                                     */
/* Web : http://www.spidboutic.fr                                             */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

package com.ael.crm.phoning;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.util.HashMap;
import java.util.Vector;
import java.util.Collections;
import java.util.Iterator;

import com.ael.crm.metier.Prospect;
import com.ael.crm.metier.FabriqueComparateurProspect;

import org.experlog.openeas.api.Session;

/**
 * Classe permettant la récupération de données en vue d'affichage dans le 
 * planing du phoning de la CRM
 * 
 * @author jm (AEL Sarl)
 *
 */
public class PlanningPhoning {
	
	// base sql des données FACT_<dossier_id>
	private String base;
	// connexion
	private Connection conn;
	
	// chaine de requête de base
	private String reqTemplate;
	// requête de jointure
	private String reqJointure;
	// chaîne de requête finale
	private StringBuffer reqProspects;
	
	// vector des identifiants de prospects
	private HashMap<String,Integer> index =  new HashMap<String,Integer>();
	private Vector<Prospect> prospects = new Vector<Prospect>();
	
	/**
	 * Constructeur privé pour objet <code>PlanningPhoning</code>
	 * 
	 * @param	session				session sous forme de chaîne identifiante
	 * @param 	base				nom de la base des données
	 */
	private PlanningPhoning(String session, String base_) {
		try {
			Session s = Session.findClient(session);
			conn = s.getConnection(null);
			base = base_;
			
			reqTemplate = "SELECT p.Id_Prospect, p.Raison_Sociale, e.Type_Evenement, r.Date_Evenement " +
			              "FROM " + base + ".PROSPECT p JOIN "  +
			              base + ".COMMUNICATION c ON p.Id_Prospect = c.Id_Prospect JOIN " +
			              base + ".RESOLUTION_COMMUNICATION rc on rc.Id_Communication = c.Id_Communication JOIN " +
			              base + ".RESOLUTION r on rc.Id_Resolution = r.Id_Resolution JOIN " +
			              base + ".EVENEMENT e on r.Id_Evenement = e.Id_Evenement";
			
			reqProspects = new StringBuffer(reqTemplate);
			
			reqJointure = " JOIN (" + reqTemplate + " WHERE p.Supprime = 0";
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Construit un objet <code>PlanningPhoning</code>
	 * 
	 * @param	session				session sous forme de chaîne identifiante
	 * @param 	base				nom de la base des données
	 * @param	demandePlaquette	booléen indiquant si la recherche porte sur les demandes de plaquette
	 */
	public PlanningPhoning(String session,
			               String base,
			               boolean demandePlaquette) {
		this(session, base);
		
		ajouterCriterePlaquette(demandePlaquette);
		ajouterCritereNonSuppression();
	}
	
	/**
	 * Construit un objet <code>PlanningPhoning</code>
	 * 
	 * @param	session				session sous forme de chaîne identifiante
	 * @param 	base				nom de la base des données
	 * @param	demandePlaquette	booléen indiquant si la recherche porte sur les demandes de plaquette
	 * @param 	typeContact			chaîne indiquant le type de contact analysé (rendez-vous, recontact)
	 * @param 	typeIntervalle		chaîne indiquant le type d'intervalle recherché pour la date des contacts
	 * @param	dateDebut			date de début de l'intervalle de recherche pour le contact
	 * @param	dateFin				date de fin de l'intervalle de recherche pour le contact
	 * @param 	ville				chaîne indiquant le motif de ville recherché
	 */
	public PlanningPhoning(String session,
            			   String base,
            			   boolean demandePlaquette,
            			   String typeContact,
            			   String typeIntervalle,
            			   long dateDebut,
            			   long dateFin) {
		this(session, base);
		
		ajouterCriterePlaquette(demandePlaquette);
		reqProspects.append(reqJointure);
		
		if (typeContact.equals("recontact"))
			reqProspects.append(" AND e.Type_Evenement = 'A recontacter'");
		else if (typeContact.equals("rdv"))
			reqProspects.append(" AND e.Type_Evenement = 'Rendez-vous'");
		
		if (typeIntervalle.equals("fourchette"))
			reqProspects.append(" AND r.Date_Evenement BETWEEN " + dateDebut + " AND " + dateFin);
		else if (typeIntervalle.equals("jour"))
			reqProspects.append(" AND r.Date_Evenement >= " + dateDebut);
		
		reqProspects.append(") CT ON p.Id_Prospect = CT.Id_Prospect");
		
		
		ajouterCritereNonSuppression();
	}

	/**
	 * Ajouter des clauses de recherche de l'évènement 'Envoyer plaquette'
	 * @param	plaquette booléen indiquant si la clause doit être ajoutée
	 */
	private void ajouterCriterePlaquette(boolean plaquette) {
		if (plaquette) {
			reqProspects.append(reqJointure);
			reqProspects.append(" AND e.Type_Evenement = 'Envoyer plaquette') PLQ ON p.Id_Prospect = PLQ.Id_Prospect");
		}
	}
	
	/**
	 * Ajouter une clause de recherche sur la ville du prospect
	 * @param	ville	chaîne donnant le motif de la ville recherchée
	 */
	public void ajouterCritereVille(String ville) {
		if (!ville.equals(""))
			reqProspects.append(" AND p.Ville LIKE '" + ville + "%'");
	}
	
	/**
	 * Ajouter une clause de recherche sur la raison sociale du prospect
	 * @param	rs	chaîne donnant le motif de la raison sociale recherchée
	 */
	public void ajouterCritereRaisonSociale(String rs) {
		if (!rs.equals(""))
			reqProspects.append(" AND p.Raison_Sociale LIKE '" + rs + "%'");
	}
	
	/**
	 * Ajouter la clause sur la non suppression du prospect
	 */
	private void ajouterCritereNonSuppression() {
		reqProspects.append(" WHERE p.Supprime = 0");
		
	}
	
	/**
	 * Ajouter la clause sur l'ordre de requete
	 * @param	cle		chaîne permettant d'évaluer l'attribut de tri
	 * @param	sens	chaîne de sens pour l'ordre (desc ou asc)
	 */
	public Vector trierProspects(String cle, String sens) {
		try {
			Collections.sort(prospects, FabriqueComparateurProspect.getComparateurProspect(cle));
			if (sens.equals("ASC"))
				Collections.reverse(prospects);
			// ré-indexer les prospects
			Iterator it = prospects.iterator();
			int i = 0;
			index.clear();
			while (it.hasNext()) {
				Prospect p = (Prospect)it.next();
				index.put(p.getIdProspect(), new Integer(i));
				i++;
			}
			return prospects;
		} catch (ClassNotFoundException cnfe) {
			return null;
		}
	}
	
	/**
	 * Générer le résultat de la requete
	 * @returns tous les prospects
	 */
	public Vector genererPlanning() {
		try {
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery(reqProspects.toString());
			Prospect p;
			String idProspect;
			int i = 0;	// index de l'enregistrement dans le résultat
			while (rs.next()) {
				idProspect = rs.getString("Id_Prospect");
				if (index.containsKey(idProspect))
					p = prospects.get(index.get(idProspect).intValue());
				else {
					p = new Prospect(idProspect, rs.getString("Raison_Sociale"));
					prospects.add(p);
					index.put(idProspect, new Integer(i));
					i++;
				}
				
				String typeEvt = rs.getString("Type_Evenement");
				
				if (typeEvt.equals("Rendez-vous"))
					p.setDateRdv(rs.getLong("Date_Evenement"));
				else if (typeEvt.equals("A recontacter"))
					p.setDateRecontact(rs.getLong("Date_Evenement"));
				else if (typeEvt.equals("Envoyer plaquette"))
					p.setDemandePlaquette(rs.getLong("Date_Evenement"));
			}
		} catch (Exception x) {
			System.out.println("genererPlanning : " + x);
			x.printStackTrace();
		}
		return prospects;
	}
	
	/**
	 * Récupérer le vecteur des prospects
	 */
	public Vector getProspects() {
		return prospects;
	}
	
	/**
	 * Récupérer le nombre de prospects
	 */
	public int getNbProspects() {
		return prospects.size();
	}
	
	/**
	 * Récupérer les valeurs d'un prospect suivant son identifiant
	 * @param idx	index du prospect dans le vecteur
	 * @return un Prospect
	 */
	public Prospect getProspectAt(int idx) {
		try {
			return prospects.get(idx);
		} catch (Exception x) {
			x.printStackTrace();
			return null;
		}
	}
	
	/**
	 * Récupérer l'identifiant d'un prospect connaissant son index dans le vecteur
	 * @param idx	index du prospect dans le vecteur
	 * @return l'identifiant du prospect indexé
	 */
	public String getIdProspectAt(int idx) {
		try {
			return prospects.get(idx).getIdProspect();
		} catch (Exception x) {
			x.printStackTrace();
			return "";
		}
	}
	
	/**
	 * Récupérer la raison sociale d'un prospect connaissant son index dans le vecteur
	 * @param idx	index du prospect dans le vecteur
	 * @return la raison sociale du prospect indexé
	 */
	public String getRaisonSocialeAt(int idx) {
		try {
			return prospects.get(idx).getRaisonSociale();
		} catch (Exception x) {
			x.printStackTrace();
			return "";
		}
	}
	
	/**
	 * Récupérer la date de l'évènement 'Envoyer Plaquette' d'un prospect connaissant son identifiant
	 * @param idx	index du prospect dans le vecteur
	 * @return la date en ms de l'évènement (1 pour la plaquette) du prospect indexé
	 */
	public long getDemandePlaquetteAt(int idx) {
		try {
			return prospects.get(idx).getDemandePlaquette();
		} catch (Exception x) {
			x.printStackTrace();
			return 0;
		}
	}
	
	/**
	 * Récupérer la date de l'évènement 'A recontacter' d'un prospect connaissant son identifiant
	 * @param idx	index du prospect dans le vecteur
	 * @return la date en ms de l'évènement du prospect indexé
	 */
	public long getDateRecontactAt(int idx) {
		try {
			return prospects.get(idx).getDateRecontact();
		} catch (Exception x) {
			x.printStackTrace();
			return 0;
		}
	}
	
	/**
	 * Récupérer la date de l'évènement 'Rendez-vous' d'un prospect connaissant son identifiant
	 * @param idx	index du prospect dans le vecteur
	 * @return la date en ms de l'évènement du prospect indexé
	 */
	public long getDateRdvAt(int idx) {
		try {
			return prospects.get(idx).getDateRdv();
		} catch (Exception x) {
			x.printStackTrace();
			return 0;
		}
	}
}
