/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

package com.ael.util;


import java.util.Calendar;
import java.util.Date;
import java.text.SimpleDateFormat;

/**
 * Classe d'origine org.opensi.util.tools.DateTime.java
 * modifiée et complétée pour AEL Sarl
 *
 */

public class DateTime {


	private Calendar dt;
	
	
	/**
	 * Instancie un <code>DateTime</code> à la date courante
	 */
	public DateTime() {
		dt = Calendar.getInstance();
	}
	
	/**
	 * Instancie un <code>DateTime</code> à la date donnée
	 * 
	 * @param	jour	jour voulu pour la date
	 * @param	mois	mois désiré
	 * @param	annee	année désirée
	 */
	public DateTime(int jour, int mois, int annee) {
		dt = Calendar.getInstance();
		dt.set(annee, mois-1, jour);
	}

	/**
	 * Instancie un <code>DateTime</code> au timestamp donné
	 * @param timeInMillis timestamp en millisecondes représentant une date depuis Epoch
	 */
	public DateTime(long timeInMillis) {
		dt = Calendar.getInstance();
		dt.setTimeInMillis(timeInMillis);
	}
	
	/**
	 * Construit une <code>Periode</code> avec une chaîne de date 
	 * au format aaaa-mm-jj ou bien jj-mm-aaaa
	 */
	public DateTime(String chaineDate) {
		dt = Calendar.getInstance();
		String[] tab = chaineDate.split("-");
		try {
			if (tab[0].length() == 4) // us format
				dt.set(Integer.parseInt(tab[0]), Integer.parseInt(tab[1]) - 1, Integer.parseInt(tab[2]));
			else if (tab[0].length() == 2) // européen
				dt.set(Integer.parseInt(tab[2]), Integer.parseInt(tab[1]) - 1, Integer.parseInt(tab[0]));
		} catch (Exception x) {
			x.printStackTrace();
			dt = null;
		}
	}
	
	/**
	 * Obtenir la date en milli secondes
	 * @return l'instance en millisecondes depuis epoch
	 */
	public long getTimeInMillis() {	
		return dt.getTimeInMillis();
	}
	
	/**
	 * Obtenir la date en milli secondes avec éléments de heure/minute/secondestime à 0
	 * @return la valeur de l'instance en millisecondes depuis epoch, 
	 * 		   avec les heures, minutes et secondes à 0
	 */
	public long getDateDebutJourneeInMillis() {
	
		Calendar c = Calendar.getInstance();
		c.clear();
		
		c.set(Calendar.YEAR, dt.get(Calendar.YEAR));
		c.set(Calendar.MONTH, dt.get(Calendar.MONTH));
		c.set(Calendar.DAY_OF_MONTH, dt.get(Calendar.DAY_OF_MONTH));
		return c.getTimeInMillis();
  }
	
	
	/**
	 * Obtenir la date a la dernière milli-seconde du jour
	 * @return la valeur de l'instance en millisecondes depuis epoch, 
	 * 		   avec les heures, minutes et secondes paramétrés à la dernière milli seconde
	 * 		   du jour
	 */
	public long getDateFinJourneeInMillis() {
	
		Calendar c = Calendar.getInstance();
		c.setTimeInMillis(getDateDebutJourneeInMillis());
		
		c.set(Calendar.HOUR_OF_DAY, 23);
		c.set(Calendar.MINUTE, 59);
		c.set(Calendar.SECOND, 59);
		c.set(Calendar.MILLISECOND, 999);
		
		return c.getTimeInMillis();		
	}
	
	
	public int getDay() { return dt.get(Calendar.DAY_OF_MONTH); }
	public int getMonth() { return dt.get(Calendar.MONTH) + 1; }
	public int getYear() { return dt.get(Calendar.YEAR); }
	
	public void setDay(int day) { dt.set(Calendar.DAY_OF_MONTH, day); }	
	public void setMonth(int month) {	dt.set(Calendar.MONTH, month-1); }	
	public void setYear(int year) { dt.set(Calendar.YEAR, year); }
	
	/**
	 * Reparamétrer l'instance avec de nouvelles valeurs de jour, mois, année
	 * 
	 * @param jour		une nouvelle valeur pour le jour
	 * @param mois		nouvelle valeur pour le mois
	 * @param annee		nouvelle valeur pour l'année
	 */
	public void setDate(int jour, int mois, int annee) {
		dt.set(annee, mois-1, jour);
	}	
	
	/**
	 * Reparamétrer l'instance avec une nouvelle valeur en milli-secondes
	 * 
	 * @param timeInMillis		une nouvelle valeur de l'instance en ms
	 */
	public void setTimeInMillis(long timeInMillis) {
		dt.setTimeInMillis(timeInMillis);
	}
	
	/**
	 * Obtenir le dernier jour du mois courant
	 * 
	 * @return le nombre du dernier jour du mois courant paramétré dans l'instance
	 */
	public int getDernierJourMois() {
		return dt.getActualMaximum(Calendar.DAY_OF_MONTH);
	}
	
	/**
	 * Obtenir le dernier jour du mois précédent
	 * 
	 * @return le nombre du dernier jour du mois précédent celui paramétré dans l'instance
	 */
	public int previousMaxDay() {
		Calendar c = Calendar.getInstance();
		c.setTimeInMillis(getTimeInMillis());
		c.set(Calendar.DAY_OF_MONTH, 1);
		c.set(Calendar.MONTH, c.get(Calendar.MONTH) - 1);
	
		return c.getActualMaximum(Calendar.DAY_OF_MONTH);
	}
	
	/** 
	 * Obtenir le dernier jour du mois suivant
	 * 
	 * @return le nombre du dernier jour du mois suivant celui paramétré dans l'instance
	 */
	public int nextMaxDay() {
		Calendar c = Calendar.getInstance();
		c.setTimeInMillis(getTimeInMillis());
		c.set(Calendar.DAY_OF_MONTH, 1);
		c.set(Calendar.MONTH, c.get(Calendar.MONTH) + 1);
	
		return c.getActualMaximum(Calendar.DAY_OF_MONTH);
	}
	
	/**
	 * Obtenir une période sur la journée paramétrée dans l'instance
	 * 
	 * @return un objet période paramétré avec le début et la fin de journée en ms
	 */
	public Periode getPeriodeJournee() {
		return new Periode(getDateDebutJourneeInMillis(), getDateFinJourneeInMillis());
	}
	
	/**
	 * Obtenir une période sur le mois paramétré dans l'instance
	 * 
	 * @return un objet période paramétré avec le début du premier jour du mois
	 *         et la fin de journée du dernier jour du mois en ms
	 */
	public Periode getPeriodeMois() {
		Periode p = new Periode();
		Calendar c = Calendar.getInstance();
		c.clear();
		
		c.set(Calendar.YEAR, dt.get(Calendar.YEAR));
		c.set(Calendar.MONTH, dt.get(Calendar.MONTH));
		c.set(Calendar.DAY_OF_MONTH, 1);
		p.setDebut(c.getTimeInMillis());
		c.set(Calendar.DAY_OF_MONTH, c.getActualMaximum(Calendar.DAY_OF_MONTH));
		c.set(Calendar.HOUR_OF_DAY, 23);
		c.set(Calendar.MINUTE, 59);
		c.set(Calendar.SECOND, 59);
		c.set(Calendar.MILLISECOND, 999);
		p.setFin(c.getTimeInMillis());
		
		return p;
	}
	
	/**
	 * Obtenir une période sur la semaine paramétrée par le jour et le mois dans l'instance
	 * 
	 * @return un objet période paramétré avec le début du premier jour de la semaine
	 *         et la fin de journée du dernier jour de celle ci
	 */
	public Periode getPeriodeSemaine() {
		Periode p = new Periode();
		Calendar c = Calendar.getInstance();
		c.clear();
		
		// Le premier jour de la semaine commence le dimanche
		int jourSemaine = dt.get(Calendar.DAY_OF_WEEK);
		
		long millis = getDateDebutJourneeInMillis();
		long millisDebutSemaine = millis - (86400000 * (jourSemaine - dt.getFirstDayOfWeek())); // 86400 s/jour
		long millisFinSemaine = millisDebutSemaine + (86400000 * 7) - 1;
		p.setDebut(millisDebutSemaine);
		p.setFin(millisFinSemaine);
		
		return p;
	}
	
	/**
	 * Obtenir une période sur l'année paramétrée par l'année de l'instance
	 * 
	 * @return un objet période paramétré avec le début du premier jour de l'année
	 *         et la fin de journée du dernier jour de celle ci
	 */
	public Periode getPeriodeAnnee() {
		Periode p = new Periode();
		Calendar c = Calendar.getInstance();
		c.clear();
		
		c.set(Calendar.YEAR, dt.get(Calendar.YEAR));
		c.set(Calendar.MONTH, 0);
		c.set(Calendar.DAY_OF_MONTH, 1);
		p.setDebut(c.getTimeInMillis());
		c.set(Calendar.MONTH, 11);
		c.set(Calendar.DAY_OF_MONTH, 31);
		c.set(Calendar.HOUR_OF_DAY, 23);
		c.set(Calendar.MINUTE, 59);
		c.set(Calendar.SECOND, 59);
		c.set(Calendar.MILLISECOND, 999);
		p.setFin(c.getTimeInMillis());
		
		return p;
	}
	
	/**
	 * Analyse une date donnée selon un patron
	 * 
	 * @param	une date sous forme de chaîne
	 * @param	le patron correspondant à la chaîne de date
	 * @return  le nombre de ms depuis epoch représentant la date analysée, 
	 *          ou -1 si pas de correspondance
	 */
	public static long parseDate(String dateStr, String pattern) {
		try {
			SimpleDateFormat sdf = new SimpleDateFormat(pattern);
			sdf.setLenient(false);
	
			Date date = sdf.parse(dateStr, new java.text.ParsePosition(0));
		
			return date.getTime();
			
		} catch(NullPointerException npe) {
			return -1;
		}
	}
	
	
	/**
	 * Obtenir une date formattée selon un pattron
	 * 
	 * @param	une date en ms
	 * @param	le patron correspondant à la chaîne de date
	 * @return  la date correspondant au nombre de ms depuis epoch analysé
	 *          formattée suivant le patron donné
	 */
	public static String formatTime(long timeInMillis, String pattern) {
	
		SimpleDateFormat sdf = new SimpleDateFormat(pattern);
		
		return sdf.format(new Date(timeInMillis));
	} 


} // fin DateTime
